import React from 'react';
import PropTypes from 'prop-types';
import { isDarkMode } from '@cloudflare/style-const';
import raf from 'raf';
import { createComponent } from '@cloudflare/style-container';
import { Icon } from '@cloudflare/component-icon';

var getBorderColor = (type, theme) => {
  switch (type) {
    case 'success':
      return theme.colors.green[5];

    case 'warning':
      return theme.colors.orange[5];

    case 'error':
      return theme.colors.red[2];

    case 'info':
    default:
      return theme.colors.blue[3];
  }
};

var getBackgroundColor = (type, theme) => {
  switch (type) {
    case 'success':
      return theme.colors.green[isDarkMode() ? 3 : 6];

    case 'warning':
      return theme.colors.orange[isDarkMode() ? 3 : 6];

    case 'error':
      return theme.colors.red[3];

    case 'info':
    default:
      return theme.colors.blue[isDarkMode() ? 3 : 6];
  }
};

var NotificationMessage = createComponent(_ref => {
  var {
    theme
  } = _ref;
  return {
    position: 'relative',
    zIndex: 2,
    color: theme.colors.white,
    display: 'flex',
    alignItems: 'center'
  };
}, 'span');
var NotificationClose = createComponent(_ref2 => {
  var {
    theme,
    type
  } = _ref2;
  return {
    position: 'absolute',
    zIndex: 3,
    top: 0,
    right: 0,
    bottom: 0,
    width: '2.5em',
    cursor: 'pointer',
    '&:hover': {
      backgroundColor: getBorderColor(type, theme)
    },
    '&::before': {
      position: 'absolute',
      top: '50%',
      left: '50%',
      transform: 'translate(-50%, -50%)'
    }
  };
}, 'span');
var NotificationContainer = createComponent(_ref3 => {
  var {
    theme,
    type
  } = _ref3;
  return {
    position: 'relative',
    padding: theme.space[2],
    verticalAlign: 'middle',
    '-webkit-text-stroke': '0',
    '-webkit-font-smoothing': 'antialiased',
    cursor: 'pointer',
    transformOrigin: 'bottom left',
    borderTopColor: getBorderColor(type, theme),
    borderBottomColor: getBorderColor(type, theme),
    borderLeftColor: getBorderColor(type, theme),
    borderRightColor: getBorderColor(type, theme),
    borderTopWidth: 1,
    borderRightWidth: 1,
    borderBottomWidth: 0,
    borderLeftWidth: 1,
    borderStyle: 'solid',
    backgroundColor: getBackgroundColor(type, theme),
    animationName: {
      '0%': {
        transform: 'scaleY(0)'
      },
      '100%': {
        transform: 'scaleY(1)'
      }
    },
    animationDuration: '200ms',
    animationTimingFunction: 'ease-in-out',
    '&::before': {
      position: 'absolute',
      top: '50%',
      left: '0.75rem',
      transform: 'translate(0, -50%)',
      lineHeight: '1',
      verticalAlign: 'middle'
    },
    '&:first-child': {
      borderTopWidth: 1
    }
  };
}, 'div');
var IconWrapper = createComponent(() => ({
  paddingRight: 6,
  display: 'flex'
}));
var NotificationWrapper = createComponent(() => ({
  display: 'flex'
}));
var CloseWrapper = createComponent(() => ({
  position: 'absolute',
  top: '50%',
  left: '50%',
  transform: 'translate(-50%,-50%)'
}));
var NotificationItem = createComponent(() => ({}), _ref4 => {
  var {
    children,
    closable,
    handleClickClose,
    type
  } = _ref4;
  var iconType;

  switch (type) {
    case 'success':
      iconType = 'ok-sign';
      break;

    case 'info':
      iconType = 'info-sign';
      break;

    case 'warning':
      iconType = 'exclamation-sign';
      break;

    case 'error':
      iconType = 'exclamation-sign';
      break;

    default:
      iconType = 'info-sign';
      break;
  }

  return /*#__PURE__*/React.createElement(NotificationWrapper, null, /*#__PURE__*/React.createElement(NotificationMessage, null, /*#__PURE__*/React.createElement(IconWrapper, null, /*#__PURE__*/React.createElement(Icon, {
    type: iconType,
    label: type ? type : 'info-sign',
    color: "white",
    size: 24
  })), children), closable && /*#__PURE__*/React.createElement(NotificationClose, {
    onClick: handleClickClose,
    type: type
  }, /*#__PURE__*/React.createElement(CloseWrapper, null, /*#__PURE__*/React.createElement(Icon, {
    type: "remove",
    label: "close",
    color: "white"
  }))));
});
NotificationItem.propTypes = {
  children: PropTypes.string,
  closable: PropTypes.bool,
  persist: PropTypes.bool,
  duration: PropTypes.number,
  active: PropTypes.bool,
  handleClickClose: PropTypes.func
};

class Notification extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      persist: this.props.persist,
      timeoutId: null
    };
  }

  componentDidMount() {
    // Read this https://stanko.github.io/react-rerender-in-component-did-mount/
    this._raf = raf(() => {
      this._raf = raf(() => {
        this._raf = null;
        this.startTimeout();
      });
    });
  }

  componentWillUnmount() {
    this.stopTimeout();
  }

  startTimeout() {
    // Don't dismiss errors if they're configured to be persisted,
    // or if we're in a testing environment
    if (this.state.persist || process.env.TESTING) {
      return;
    }

    this.stopTimeout();
    var timeoutId = setTimeout(() => this.props.onClose(), this.props.delay);
    this.setState({
      timeoutId
    });
  }

  stopTimeout() {
    if (this._raf) {
      raf.cancel(this._raf);
      this._raf = null;
    }

    if (this.state.timeoutId) {
      clearTimeout(this.state.timeoutId);
      this.setState({
        timeoutId: null
      });
    }
  }

  handleMouseEnter() {
    this.stopTimeout();
  }

  handleMouseLeave() {
    this.startTimeout();
  }

  handleClickClose() {
    this.props.onClose();
  }

  handleClick() {
    this.stopTimeout();
    this.setState({
      persist: true
    });
  }

  render() {
    return /*#__PURE__*/React.createElement(NotificationContainer, {
      role: "alert",
      onClick: this.handleClick.bind(this),
      onMouseEnter: this.handleMouseEnter.bind(this),
      onMouseLeave: this.handleMouseLeave.bind(this),
      type: this.props.type
    }, /*#__PURE__*/React.createElement(NotificationItem, {
      closable: this.props.closable,
      handleClickClose: this.handleClickClose.bind(this),
      duration: this.props.delay,
      persist: this.state.persist,
      active: !!this.state.timeoutId,
      type: this.props.type
    }, this.props.message));
  }

}

Notification.propTypes = {
  type: PropTypes.oneOf(['success', 'error', 'info', 'warning']),
  message: PropTypes.string.isRequired,
  closable: PropTypes.bool,
  delay: PropTypes.number,
  persist: PropTypes.bool,
  onClose: PropTypes.func.isRequired
};
Notification.defaultProps = {
  closable: true,
  delay: 4000,
  persist: false,
  type: 'info'
};
export default Notification;